import flask
from viggocore.common import exception, manager, utils
from viggocore.subsystem.log_system.functions import (
    entity_create)


class Manager(manager.CommonManager):

    def __init__(self, driver):
        super(Manager, self).__init__(driver)
        self.create = entity_create.Create(self)

    def _get_user_informado(self, domain_id, user_name):
        users = self.api.users().list(
            domain_id=domain_id, name=user_name)
        if len(users) == 1:
            return users[0]
        else:
            raise exception.BadRequest('Esse usuário não existe.')

    def valida_user(self, user=None, **kwargs):
        domain_id = kwargs.get('domain_id', None)
        if domain_id is None:
            raise exception.BadRequest('O domain_id é obrigatório.')
        user_name = kwargs.get('user_name', None)
        # valida se o usuário informado existe
        if user is None:
            user = self._get_user_informado(domain_id, user_name)

        # pega esta informação para preencher a tabela de
        # log_alteracao_custo_pedido
        self.user_id = user.id

        # valida se a senha informada bate com a senha do usuário encontrado
        password_informada = kwargs.get('user_password', 'None')
        if user.password != password_informada:
            raise exception.BadRequest('A senha informada está incorreta.')

    def _get_tag(self):
        return ' - '.join([flask.request.url, flask.request.method])

    def _get_domain_id(self, entity, domain_id=None):
        if domain_id is None:
            return (
                entity.id if entity.individual() == 'domain'
                else entity.domain_id)
        return domain_id

    def _get_user_info(self, user):
        user_id = None
        user_name = None

        if user is not None:
            user_id = user.id
            user_name = user.name

        return (user_id, user_name)

    def _get_kwargs_info(self, user_name, **kwargs):
        """
        Essa função vai pegar as informações passadas pelo client na requisição
        """
        # se não for passado um "user_name" na requisição ai usará o
        # do usuário do token mesmo
        user_name = kwargs.get('user_name', user_name)
        user_password = kwargs.get('user_password', None)
        justificativa = kwargs.get('justificativa', None)

        return (user_name, user_password, justificativa)

    def criar_log_system_ativacao(self, session, result, user=None,
                                  domain_id=None, **kwargs):
        if len(result) > 0:
            tag = self._get_tag()
            entity = result[0]

            domain_id = self._get_domain_id(entity=entity, domain_id=domain_id)
            user_id, user_name = self._get_user_info(user=user)
            user_name, user_password, justificativa = (
                self._get_kwargs_info(user_name=user_name, **kwargs))

            entidade_ids = ','.join([r.id for r in result])
            json_posterior = {
                "active": entity.active
            }
            log_alteracao_dict = {
                "tag": tag,
                "domain_id": domain_id,
                "user_id": user_id,
                "user_name": user_name,
                "user_password": user_password,
                "justificativa": justificativa,
                "entidade_ids": entidade_ids,
                "entidade_nome": entity.individual(),
                "json_anterior": utils.to_json({}),
                "json_posterior": utils.to_json(json_posterior)
            }
            self.create(session=session, **log_alteracao_dict)

    def criar_log_system(self, session, entity, json_anterior,
                         json_posterior, user=None, domain_id=None, **kwargs):
        tag = self._get_tag()

        domain_id = self._get_domain_id(entity=entity, domain_id=domain_id)
        user_id, user_name = self._get_user_info(user=user)
        user_name, user_password, justificativa = (
            self._get_kwargs_info(user_name=user_name, **kwargs))

        log_alteracao_dict = {
            "tag": tag,
            "domain_id": domain_id,
            "user_id": user_id,
            "user_name": user_name,
            "user_password": user_password,
            "justificativa": justificativa,
            "entidade_ids": entity.id,
            "entidade_nome": entity.individual(),
            "json_anterior": utils.to_json(json_anterior),
            "json_posterior": utils.to_json(json_posterior)
        }
        self.create(session=session, **log_alteracao_dict)



import os
from viggocore.common.subsystem.driver import Driver
from viggocore.common.subsystem import operation
from viggocore.common import exception
from viggocore.common.subsystem.manager_functions import (
    export_pdf, make_exportar_xlsx)


DATA_DOWNLOAD_FOLDER = 'data/<id>/download'


class ExportXlsx(operation.List):
    def pre(self, **kwargs):
        self.title = kwargs.get('title', 'Exportação em XLSX')
        self.file_title = kwargs.get('file_title', 'exportacao_xlsx')
        return True

    def do(self, session, **kwargs):
        filters = kwargs.get('filters', {})
        columns = kwargs.get('columns', [])

        entities = self.manager.list(session=session, **filters)
        if type(entities) is tuple:
            entities = entities[0]

        if len(entities) > 0:
            download_folder, file_name = \
                self.manager.make_export_xlsx(
                    entities=entities,
                    columns=columns,
                    title=self.title,
                    file_title=self.file_title
                )
        else:
            raise exception.BadRequest(
                'Não existe nada para essa filtragem.')

        return download_folder, file_name


class Manager(object):

    def __init__(self, driver: Driver) -> None:
        self.driver = driver

        self.create = operation.Create(self)
        self.get = operation.Get(self)
        self.list = operation.List(self)
        self.update = operation.Update(self)
        self.delete = operation.Delete(self)
        # NOTE(samueldmq): what do we use this for ?
        self.count = operation.Count(self)
        self.list_multiple_selection = operation.ListMultipleSelection(self)
        self.activate_or_deactivate_multiple_entities = operation.\
            ActivateOrDeactivateMultipleEntities(self)
        self.export_xlsx = ExportXlsx(self)
        self.make_export_xlsx = make_exportar_xlsx.MakeExportarXsls(self)
        self.export_pdf = export_pdf.ExportPdf(self)

    def init_query(self, session, order_by, resource):
        raise exception.BadRequest(
            f'Method _init_query() not implemented for {resource.__name__}.')

    def valid_dinamic_order_by(self, order_by):
        result = False
        count_points = order_by.count('.')
        if ('.' in order_by and count_points == 1) or count_points == 0:
            result = True
        elif count_points > 1:
            raise exception.BadRequest(
                'order_by item cannot have more than one point.')
        return result

    def get_multiple_selection_ids(self, entities):
        try:
            return [entity.id for entity in entities]
        except Exception:
            raise exception.BadRequest(
                'id is not an attribute of this entity')

    def aplicar_filtro_status_periodo_data(self, query, resource, **kwargs):
        return self.driver._aplicar_filtro_status_periodo_data(
            query, resource, **kwargs)

    @classmethod
    def _get_download_folder(cls, id):
        data_download_folder = DATA_DOWNLOAD_FOLDER.replace('<id>', id)
        data_download_folder = os.environ.get('VIGGOCORE_FILE_DIR',
                                              data_download_folder)
        if not os.path.isabs(data_download_folder):
            data_download_folder = os.path.join(os.getcwd(),
                                                data_download_folder)
        return data_download_folder

    @classmethod
    def get_download_folder(cls, id):
        download_folder = cls._get_download_folder(id)
        if not os.path.exists(download_folder):
            os.makedirs(download_folder)
        return download_folder



# estilo de tabela colorida (usado no relatório de vendas)
STYLE = """
<style type="text/css">
    .div-table {
        display: flex;
        flex-direction: column;
    }
    .table {
        width: 100%;
        height: auto;
        margin-top: 1px;
        border-collapse: collapse;
        text-align: center;
    }
    .tr-table-top {
        border-bottom: 0.5px solid gray;
        border-top: 0.5px solid gray;
    }
    .tr-table-center {
        border-top: 0px solid gray;
        border-bottom: 0px solid gray;
    }
    .tr-table-bottom {
        border-bottom: 0px solid gray;
    }
    .tr-table-right {
        border-right: 0.5px solid gray;
    }
    .tr-table-left {
        border-left: 0.5px solid gray;
    }
    .tr-table-color{
        background-color: #deebf7;
    }
    .td1-table {
        text-align: left;
        font-size: 9px;
        padding-left: 30px;
        padding-right: 0px !important;
    }
    .td2-table {
        text-align: right;
        font-size: 9px;
        padding-top: 1px;
        padding-bottom: 1px;
    }
    .td3-table {
        text-align: center;
        font-size: 9px;
        padding-top: 1px;
        padding-bottom: 1px;
    }
</style>
"""  # noqa



import flask

from viggocore.common import exception, utils
from viggocore.common.subsystem import controller


class CommonController(controller.Controller):

    # TODO(JorgeSilva): melhorar essa função para diminuir a complexidade da
    # função
    def _get_include_dicts_vex(self, kwargs):  # noqa: C901
        retorno = {}
        includes = kwargs.get('include', None)
        if includes is not None:
            for include in includes.split(','):
                if '.' in include:
                    # Variável aux serve para dar um include em um include.
                    # Exemplo: em uma consulta de pedido eu dou um include em
                    # cliente e quero os campos de parceiro também então o
                    # include de cliente fica: cliente.parceiro assim ele vai
                    # incluir o parceiro em cliente e o cliente em pedido
                    aux = include.split('.')
                    if len(aux) == 2:
                        if aux[0] not in retorno:
                            retorno.update({aux[0]: {aux[1]: {}}})
                        else:
                            retorno[aux[0]].update({aux[1]: {}})
                    elif len(aux) == 3:
                        if aux[0] not in retorno:
                            retorno.update({aux[0]: {aux[1]: {aux[2]: {}}}})
                        elif aux[1] not in retorno[aux[0]]:
                            retorno[aux[0]].update({aux[1]: {aux[2]: {}}})
                        else:
                            retorno[aux[0]][aux[1]].update({aux[2]: {}})
                    elif len(aux) == 4:
                        if aux[0] not in retorno:
                            retorno.update({aux[0]: {aux[1]: {aux[2]: {aux[3]: {}}}}})  # noqa: E501
                        elif aux[1] not in retorno[aux[0]]:
                            retorno[aux[0]].update({aux[1]: {aux[2]: {aux[3]: {}}}})  # noqa: E501
                        elif aux[2] not in retorno[aux[0]][aux[1]]:
                            retorno[aux[0]][aux[1]].update({aux[2]: {aux[3]: {}}})  # noqa: E501
                        else:
                            retorno[aux[0]][aux[1]][aux[2]].update(
                                {aux[3]: {}})
                    elif len(aux) == 5:
                        if aux[0] not in retorno:
                            retorno.update({aux[0]: {aux[1]: {aux[2]: {aux[3]: {aux[4]: {}}}}}})  # noqa: E501
                        elif aux[1] not in retorno[aux[0]]:
                            retorno[aux[0]].update({aux[1]: {aux[2]: {aux[3]: {aux[4]: {}}}}})  # noqa: E501
                        elif aux[2] not in retorno[aux[0]][aux[1]]:
                            retorno[aux[0]][aux[1]].update({aux[2]: {aux[3]: {aux[4]: {}}}})  # noqa: E501
                        elif aux[3] not in retorno[aux[0]][aux[1]][aux[2]]:
                            retorno[aux[0]][aux[1]][aux[2]].update({aux[3]: {aux[4]: {}}})  # noqa: E501
                        else:
                            retorno[aux[0]][aux[1]][aux[2]][aux[3]].update({aux[4]: {}})  # noqa: E501
                    elif len(aux) == 6:
                        if aux[0] not in retorno:
                            retorno.update({aux[0]: {aux[1]: {aux[2]: {aux[3]: {aux[4]: {}}}}}})  # noqa: E501
                        elif aux[1] not in retorno[aux[0]]:
                            retorno[aux[0]].update({aux[1]: {aux[2]: {aux[3]: {aux[4]: {}}}}})  # noqa: E501
                        elif aux[2] not in retorno[aux[0]][aux[1]]:
                            retorno[aux[0]][aux[1]].update({aux[2]: {aux[3]: {aux[4]: {}}}})  # noqa: E501
                        elif aux[3] not in retorno[aux[0]][aux[1]][aux[2]]:
                            retorno[aux[0]][aux[1]][aux[2]].update({aux[3]: {aux[4]: {}}})  # noqa: E501
                        elif aux[4] not in retorno[aux[0]][aux[1]][aux[2]][aux[3]]:  # noqa: E501
                            retorno[aux[0]][aux[1]][aux[2]][aux[3]].update({aux[4]: {aux[5]: {}}})  # noqa: E501
                        else:
                            retorno[aux[0]][aux[1]][aux[2]][aux[3]].update({aux[4]: {}})  # noqa: E501
                else:
                    retorno.update({include: {}})
        return retorno

    def list(self):
        filters = self._filters_parse()

        try:
            filters = self._parse_list_options(filters)
            (entities, total_rows) = self.manager.list(**filters)

            page = filters.get('page', None)
            page_size = filters.get('page_size', None)
        except exception.ViggoCoreException as exc:
            return flask.Response(response=exc.message,
                                  status=exc.status)
        except ValueError:
            raise exception.BadRequest('page or page_size is invalid')

        collection = self._entities_to_dict(
            entities, self._get_include_dicts_vex(filters))

        response = {self.collection_wrap: collection}

        if total_rows is not None:
            response.update({'pagination': {'page': int(page),
                                            'page_size': int(page_size),
                                            'total': total_rows}})

        return flask.Response(response=utils.to_json(response),
                              status=200,
                              mimetype="application/json")

    def get(self, id):
        filters = self._filters_parse()

        try:
            entity = self.manager.get(id=id)

            include_dicts = self._get_include_dicts_vex(filters)

            entity_dict = entity.to_dict(include_dict=include_dicts)
        except exception.ViggoCoreException as exc:
            return flask.Response(response=exc.message,
                                  status=exc.status)

        response = {self.resource_wrap: entity_dict}

        return flask.Response(response=utils.to_json(response),
                              status=200,
                              mimetype="application/json")



import uuid
from viggocore.common.subsystem.pagination import Pagination
from viggocore.common.subsystem.entity import DATE_FMT
from datetime import datetime as datetime1
import datetime as datetime2
from typing import Any, Type

from viggocore.common import exception
from sqlalchemy import func, or_, and_
from sqlalchemy.orm import exc
from sqlalchemy.sql import text

from viggocore.common.subsystem.transaction_manager import TransactionManager


class Driver(object):

    def __init__(self, resource: Type[Any],
                 transaction_manager: TransactionManager) -> None:
        self.resource = resource
        self.transaction_manager = transaction_manager

    def removeId(self, entity_aux):
        new_id = uuid.uuid4().hex

        if entity_aux.get('id') is not None:
            new_id = entity_aux.pop('id')

        return new_id

    def instantiate(self, **kwargs):
        try:
            embedded = {}
            for attr in self.resource.embedded():
                if attr not in kwargs:
                    raise Exception(
                        f'O campo embedded {attr} é obrigatório em ' +
                        f'{self.resource.__name__}.')
                embedded.update({attr: kwargs.pop(attr)})

            instance = self.resource(**kwargs)

            for attr in embedded:
                value = embedded[attr]
                var = getattr(self.resource, attr)
                # TODO(samueldmq): is this good enough? should we discover it?
                mapped_attr = {self.resource.individual() + '_id': instance.id}
                if isinstance(value, list):
                    setattr(instance, attr, [var.property.mapper.class_(
                        id=self.removeId(ref), **dict(ref, **mapped_attr))
                        for ref in value])
                else:
                    # TODO(samueldmq): id is inserted here. it is in the
                    # manager for the entities. do it all in the resource
                    # contructor
                    setattr(instance, attr, var.property.mapper.class_(
                        id=uuid.uuid4().hex, **dict(value, **mapped_attr)))
        except Exception as exec:
            # TODO(samueldmq): replace with specific exception
            message = ''.join(exec.args)
            raise exception.BadRequest(message)

        return instance

    def create(self, entity, session):
        if not entity.is_stable():
            raise exception.PreconditionFailed()
        session.add(entity)
        session.flush()

    def update(self, entity, data, session):
        # try:
        #     entity = self.get(id, session)
        # except exc.NoResultFound:
        #     raise exception.NotFound()

        for attr in self.resource.embedded():
            if attr in data:
                value = data.pop(attr)
                var = getattr(self.resource, attr)
                # TODO(samueldmq): is this good enough? should we discover it?
                mapped_attr = {self.resource.individual() + '_id': id}
                if isinstance(value, list):
                    setattr(entity, attr, [var.property.mapper.class_(
                        id=self.removeId(ref), **dict(ref, **mapped_attr))
                        for ref in value])
                else:
                    # TODO(samueldmq): id is inserted here. it is in the
                    # manager for the entities. do it all in the resource
                    # contructor
                    setattr(entity, attr, var.property.mapper.class_(
                        id=uuid.uuid4().hex, **dict(value, **mapped_attr)))

        for key, value in data.items():
            if hasattr(entity, key):
                try:
                    setattr(entity, key, value)
                except AttributeError:
                    raise exception.BadRequest(
                        f'Error! The attribute {key} is read only')
            else:
                raise exception.BadRequest(
                    f'Error! The attribute {key} not exists')

        if not entity.is_stable():
            raise exception.PreconditionFailed()
        session.flush()
        return entity

    def delete(self, entity, session):
        session.delete(entity)
        session.flush()

    def get(self, id, session):
        try:
            query = session.query(self.resource).filter_by(id=id)
            result = query.one()
        except exc.NoResultFound:
            raise exception.NotFound()

        return result

    def list(self, session, **kwargs):
        if 'query' in kwargs.keys():
            query = kwargs.pop('query', None)
            pagination = Pagination.get_pagination(self.resource, **kwargs)

            query = self.apply_filters(query, self.resource, **kwargs)
            pagination.adjust_dinamic_order_by(self.resource)
            query = self.apply_pagination(query, pagination)

            result = query.all()
            result = list(map(lambda x: x[0], result))
            return result
        else:
            query = session.query(self.resource)

            pagination = Pagination.get_pagination(self.resource, **kwargs)

            query = self.apply_filters(query, self.resource, **kwargs)
            query = self.apply_pagination(query, pagination)

            result = query.all()
            return result

    def count(self, session, **kwargs):
        try:
            # TODO(JogeSilva): improve filtering so as not to ignore parameters
            # that are attributes of an entity to include
            query = session.query(self.resource.id)
            rows = self.apply_filters(query, self.resource, **kwargs).count()
            result = rows
        except exc.NoResultFound:
            raise exception.NotFound()

        return result

    def activate_or_deactivate_multiple_entities(self, session, **kwargs):
        updated_at = kwargs.pop('updated_at', None)
        updated_by = kwargs.pop('updated_by', None)
        active = kwargs.pop('active', None)

        entities = self.list_multiple_selection(session, **kwargs)
        key = 'active'

        for entity in entities:
            if hasattr(entity, key):
                setattr(entity, key, active)
                if updated_at is not None:
                    setattr(entity, 'updated_at', updated_at)
                if updated_by is not None:
                    setattr(entity, 'updated_by', updated_by)
            else:
                raise exception.BadRequest(
                    f'Error! The attribute {key} not exists.')

            if not entity.is_stable():
                raise exception.PreconditionFailed()
        session.flush()
        return entities

    def _parse_list_options(self, filters):
        key = filters.pop('list_options', None)
        _filters = filters.copy()
        options = {
            'ACTIVE_ONLY': True,
            'INACTIVE_ONLY': False
        }
        if key in options.keys():
            _filters['active'] = options[key]
        return _filters

    def list_multiple_selection(self, session, **kwargs):
        '''
            fields that can be passed in kwargs:

            dict_compare
            resource
            query

            the possibility of passing these fields as parameters makes
            the function generic.
        '''

        dict_compare = kwargs.pop('dict_compare', {})
        only_first_column = kwargs.pop('only_first_column', False)
        multiple_selection = kwargs.get('multiple_selection', None)
        if multiple_selection is None:
            raise exception.BadRequest('multiple_selection is required.')

        resource_filtro = self.resource
        resource = kwargs.pop('resource', None)
        if resource is not None:
            resource_filtro = resource

        if 'query' in kwargs.keys():
            query = kwargs.pop('query', None)
        else:
            query = session.query(resource_filtro)

        selected_list = multiple_selection.get('selected_list', [])
        unselected_list = multiple_selection.get('unselected_list', [])

        if len(selected_list) > 0:
            query = query.filter(resource_filtro.id.in_(selected_list))
        else:
            if len(unselected_list) > 0:
                query = query.filter(
                    resource_filtro.id.not_in(unselected_list))
            if 'list_options' in kwargs.keys():
                kwargs = self._parse_list_options(kwargs)
            query = self.apply_filters(query, resource_filtro, **kwargs)
            query = self.apply_filters_includes(query, dict_compare, **kwargs)
            query = self.apply_filter_de_ate_with_timezone(
                resource_filtro, query, **kwargs)

        query = query.distinct()
        result = query.all()

        if only_first_column:
            result = list(map(lambda x: x[0], result))

        return result

    def _is_id(self, k):
        k = k.split('_')[-1]
        return k == 'id'

    def apply_filter_individual(self, query, resource, k, v):
        # verifica se o valor é do tipo string
        isinstance_aux = isinstance(v, str)
        # pega o atributo da classe para verificar se o campo da
        # classe é do tipo string
        attr = getattr(resource, k)

        if self._is_id(k):
            query = query.filter(getattr(resource, k) == v)
        elif k == 'tag':
            values = v
            if len(v) > 0 and v[0] == '#':
                values = v[1:]
            values = values.split(',')
            filter_tags = []
            for value in values:
                filter_tags.append(
                    getattr(resource, k)
                    .like('%#' + str(value) + ' %'))
            query = query.filter(or_(*filter_tags))
        elif isinstance_aux and '%' in v:
            normalize = func.viggocore_normalize
            query = query.filter(
                normalize(getattr(resource, k)).ilike('%' + normalize(v)))
        elif isinstance_aux and attr.type.python_type is str:
            normalize = func.viggocore_normalize
            query = query.filter(
                normalize(getattr(resource, k)) == normalize(v))
        else:
            query = query.filter(getattr(resource, k) == v)

        return query

    def apply_filter_list(self, query, resource, k, v):
        # verifica se o valor é do tipo string
        isinstance_aux = isinstance(v[0], str)
        # pega o atributo da classe para verificar se o campo da
        # classe é do tipo string
        attr = getattr(resource, k)

        # verifica o tipo do primeiro campo da lista
        if self._is_id(k):
            query = query.filter(getattr(resource, k).in_(v))
        elif isinstance_aux and '%' in v[0]:
            normalize = func.viggocore_normalize
            # monta uma lista com os filtros para aplicar o "or"
            filtro = [
                normalize(getattr(resource, k)).ilike('%' + normalize(v_aux))
                for v_aux in v]
            # aplica os filtros
            query = query.filter(or_(*filtro))
        elif isinstance_aux and attr.type.python_type is str:
            normalize = func.viggocore_normalize
            # monta uma lista com os filtros para aplicar o "or"
            filtro = [
                normalize(getattr(resource, k)) == normalize(v_aux)
                for v_aux in v]
            # aplica os filtros
            query = query.filter(or_(*filtro))
        else:
            # se for tipo numérico ou outro tipo, aplica o in sem normalize
            query = query.filter(getattr(resource, k).in_(v))

        return query

    def apply_filters(self, query, resource, **kwargs):
        for k, v in kwargs.items():
            if hasattr(resource, k):
                # verifica se não foi passada uma lista de valores
                if not isinstance(v, list):
                    query = self.apply_filter_individual(query, resource, k, v)
                else:
                    query = self.apply_filter_list(query, resource, k, v)

        return query

    def apply_pagination(self, query, pagination: Pagination):
        if (pagination.order_by is not None):
            query = query.order_by(text(pagination.order_by))

        if pagination.page_size is not None:
            query = query.limit(pagination.page_size)
            if pagination.page is not None:
                query = query.offset(pagination.page * pagination.page_size)

        return query

    def apply_filters_includes(self, query, dict_compare, **kwargs):
        for k, v in kwargs.items():
            if '.' in k:
                # attribute é o campo da classe do include que deve ser filtrado
                attribute = k.split('.')[-1]
                # individual é a chave que irá identificar qual resource usar
                individual = k.split('.')[0]
                # pefa o resource pelo campo "individual"
                resource = dict_compare.get(individual, None)
                # se resource não for encontrado retorna a query sem filtrar
                if resource is not None and hasattr(resource, attribute):
                    # verifica se não foi passada uma lista de valores
                    if not isinstance(v, list):
                        query = self.apply_filter_individual(
                            query, resource, attribute, v)
                    else:
                        query = self.apply_filter_list(
                            query, resource, attribute, v)

        return query

    def with_pagination(self, **kwargs):
        require_pagination = kwargs.get('require_pagination', False)
        page = kwargs.get('page', None)
        page_size = kwargs.get('page_size', None)

        if None not in [page, page_size] and require_pagination is True:
            return True
        return False

    def __isdate(self, data, format="%Y-%m-%d"):
        res = True
        try:
            res = bool(datetime1.strptime(data, format))
        except ValueError:
            res = False
        return res

    def __get_day_and_next_day(self, data, format="%Y-%m-%d"):
        day = datetime1.strptime(data, format)
        next_day = day + datetime2.timedelta(days=1)
        return (day, next_day)

    def _convert_de_ate(self, **kwargs):
        de = kwargs.get('de', None)
        ate = kwargs.get('ate', None)
        inicio = None
        fim = None

        if de and ate:
            try:
                inicio = datetime1.strptime(de.replace(' ', '+'), '%Y-%m-%d%z')
                fim = datetime1.strptime(ate.replace(' ', '+'), '%Y-%m-%d%z') \
                    + datetime2.timedelta(days=1)

            except Exception:
                inicio = datetime1.strptime(de, DATE_FMT)
                fim = datetime1.strptime(ate, DATE_FMT) +\
                    datetime2.timedelta(days=1)

        return (inicio, fim)

    def apply_filter_de_ate_with_timezone(
            self, resource, query, **kwargs):
        attribute = kwargs.get('attribute', 'created_at')
        (de, ate) = self._convert_de_ate(**kwargs)
        if de and ate:
            if hasattr(resource, attribute):
                query = query.filter(
                    and_(getattr(resource, attribute) >= de,
                         getattr(resource, attribute) < ate))
        return query

    def _aplicar_filtro_status_periodo_data(self, query, resource, **kwargs):
        evento_status = kwargs.get('evento_status', None)
        evento_status_dh_de = kwargs.get('evento_status_dh_de', None)
        evento_status_dh_ate = kwargs.get('evento_status_dh_ate', None)
        campos = [evento_status, evento_status_dh_de, evento_status_dh_ate]
        if None not in campos:
            data = {
                'de': evento_status_dh_de,
                'ate': evento_status_dh_ate
            }
            (de, ate) = self.manager._convert_de_ate(is_date=False, **data)
            if de and ate:
                query = query.filter(
                    and_(resource.status_dh >= de,
                         resource.status_dh < ate,
                         resource.status == evento_status))
        return query



QUERY = """
SELECT tabela, pg_size_pretty(pg_total_relation_size(esq_tab)) AS tamanho_total
FROM (SELECT tablename AS tabela,
	schemaname AS esquema,
	schemaname||'.'||tablename AS esq_tab
FROM pg_catalog.pg_tables
WHERE schemaname NOT
IN ('pg_catalog', 'information_schema', 'pg_toast') ) AS x
ORDER BY tabela;
"""  # noqa


from viggocore.common import manager, exception
from viggocore.common.subsystem import operation
from viggocore.common.subsystem.pagination import Pagination
from viggocore.subsystem.capability_functionality.resource \
    import CapabilityFunctionality
from viggocore.subsystem.policy_functionality.resource \
    import PolicyFunctionality
from viggocore.subsystem.functionality.resource \
    import Functionality
from sqlalchemy import and_


class GetAvailableFunctionalitys(operation.List):

    def pre(self, **kwargs):
        self.role_id = kwargs.get('role_id', None)
        if self.role_id is None:
            raise exception.BadRequest('role_id is required')
        self.application_id = kwargs.get('application_id', None)
        if self.application_id is None:
            raise exception.BadRequest('application_id is required')
        return super().pre(**kwargs)

    def do(self, session, **kwargs):
        query = session.query(CapabilityFunctionality, Functionality). \
            join(Functionality,
                 CapabilityFunctionality.functionality_id == Functionality.id
                 ). \
            join(PolicyFunctionality,
                 and_(PolicyFunctionality.capability_functionality_id == CapabilityFunctionality.id,  # noqa
                      PolicyFunctionality.role_id == self.role_id),
                 isouter=True). \
            filter(
                CapabilityFunctionality.application_id == self.application_id)
        query = query.filter(PolicyFunctionality.role_id.is_(None))
        query = self.manager.apply_filters(
            query, CapabilityFunctionality, **kwargs)
        query = query.distinct()

        dict_compare = {'functionality.': CapabilityFunctionality,
                        'policy_functionality.': PolicyFunctionality}
        query = self.manager.apply_filters_includes(
            query, dict_compare, **kwargs)
        query = query.distinct()

        total_rows = None
        if self.manager.with_pagination(**kwargs):
            total_rows = query.count()

        pagination = Pagination.get_pagination(Functionality, **kwargs)
        if pagination.order_by is not None:
            pagination.adjust_order_by(Functionality)
        query = self.driver.apply_pagination(query, pagination)
        result = query.all()
        response = [r[0] for r in result]

        return (response, total_rows)


class GetSelectedFunctionalitys(operation.List):

    def pre(self, **kwargs):
        self.role_id = kwargs.get('role_id', None)
        if self.role_id is None:
            raise exception.BadRequest('role_id is required')
        self.application_id = kwargs.get('application_id', None)
        if self.application_id is None:
            raise exception.BadRequest('application_id is required')
        return super().pre(**kwargs)

    def do(self, session, **kwargs):
        query = session.query(CapabilityFunctionality, PolicyFunctionality.id,
                              Functionality). \
            join(Functionality,
                 CapabilityFunctionality.functionality_id == Functionality.id
                 ). \
            join(PolicyFunctionality,
                 and_(PolicyFunctionality.capability_functionality_id == CapabilityFunctionality.id,  # noqa
                      PolicyFunctionality.role_id == self.role_id)). \
            filter(
                CapabilityFunctionality.application_id == self.application_id)
        query = self.manager.apply_filters(
            query, CapabilityFunctionality, **kwargs)
        query = query.distinct()

        dict_compare = {'functionality.': Functionality,
                        'policy_functionality.': PolicyFunctionality}
        query = self.manager.apply_filters_includes(
            query, dict_compare, **kwargs)
        query = query.distinct()

        total_rows = None
        if self.manager.with_pagination(**kwargs):
            total_rows = query.count()

        pagination = Pagination.get_pagination(Functionality, **kwargs)
        if pagination.order_by is not None:
            pagination.adjust_order_by(Functionality)
        query = self.driver.apply_pagination(query, pagination)
        result = query.all()
        response = [(r[0], r[1]) for r in result]

        return (response, total_rows)


class Manager(manager.CommonManager):

    def __init__(self, driver):
        super(Manager, self).__init__(driver)
        self.get_available_functionalities = GetAvailableFunctionalitys(self)
        self.get_selected_functionalities = GetSelectedFunctionalitys(self)



STYLE = """
<style type="text/css">
  .div-table {
    display: flex;
    flex-direction: column;
  }

  .table {
    width: 100%;
    height: auto;
    margin-top: 10px;
    border-collapse: collapse;
    border: 0.5px solid gray;
  }

  .tr-table {
    text-align: center;
    font-size: 10px;
    padding-left: 10px;
    border: 0.5px solid gray;
  }

  .td-table {
    text-align: center;
    font-size: 10px;
    padding-left: 10px;
    border: 0.5px solid gray;
  }

  .div-header {
    display: flex;
    flex-direction: row;
    justify-content: space-between;
    align-items: center;
  }

  .sub-header-info {
    display: flex;
    flex-direction: column;
    justify-content: end;
    width: 100%;
  }

  .sub-header-text {
    display: flex;
    flex-direction: row;
    align-items: center;
    justify-content: start;
  }

  .sub-header-info-label {
    width: 20%;
    text-align: start;
  }

  .div-sub-header {
    border-top: 1px solid gray;
  }
</style>"""  # noqa


from viggocore.common import subsystem
from viggocore.subsystem.alembic_version_history import resource, router

subsystem = subsystem.Subsystem(resource=resource.AlembicVersionHistory,
                                router=router.Router)



from datetime import datetime
from alembic.script import ScriptDirectory
from alembic import util
from alembic.runtime.environment import EnvironmentContext
from alembic.runtime import migration
import flask_migrate
from flask.globals import current_app


def criar_tabela_alembic_version_history(manager):
    manager.driver.transaction_manager.session.execute(
    """
        CREATE TABLE IF NOT EXISTS alembic_version_history (
            id bpchar(32) NOT NULL,
            active bool NULL,
            created_at timestamp NULL,
            created_by bpchar(32) NULL,
            updated_at timestamp NULL,
            updated_by bpchar(32) NULL,
            tag varchar(1000) NULL,
            ordem SERIAL PRIMARY KEY,
            titulo varchar(200) NOT NULL,
            anterior_id varchar(50) NULL,
            atual_id varchar(50) NOT NULL,
            data_criacao timestamp NOT NULL,
            data_execucao timestamp NOT NULL,
            tipo varchar(50) NOT NULL DEFAULT 'UPGRADE'
        );
    """)  # noqa


# alteração de upgrade
def dbupgrade(directory=None, revision='head', sql=False, tag=None,
              x_arg=None):
    """Upgrade to a later version"""
    config = flask_migrate.current_app.extensions['migrate']\
        .migrate.get_config(directory, x_arg=x_arg)
    # realiza o upgrade
    upgrade(config, revision, sql=sql, tag=tag)


def upgrade(config, revision, sql=False, tag=None):
    """Upgrade to a later version.

    :param config: a :class:`.Config` instance.

    :param revision: string revision target or range for --sql mode

    :param sql: if True, use ``--sql`` mode

    :param tag: an arbitrary "tag" that can be intercepted by custom
    ``env.py`` scripts via the :meth:`.EnvironmentContext.get_tag_argument`
    method.

    """
    # cria a tabela alembic_version_history caso não exista
    manager = current_app.api_handler.api().alembic_version_historys()
    criar_tabela_alembic_version_history(manager)

    # variável responsável por armazenar as informações que serão usadas
    # para preencher o histórico no banco
    histories = []
    script = ScriptDirectory.from_config(config)

    starting_rev = None
    if ":" in revision:
        if not sql:
            raise util.CommandError("Range revision not allowed")
        starting_rev, revision = revision.split(":", 2)

    def upgrade(rev, context):
        response, histories_aux = _upgrade_revs(script, revision, rev)
        histories.extend(histories_aux)
        return response

    with EnvironmentContext(
        config,
        script,
        fn=upgrade,
        as_sql=sql,
        starting_rev=starting_rev,
        destination_rev=revision,
        tag=tag,
    ):
        script.run_env()

    # preenche a tabela de histórico das migrações
    for history in histories:
        manager.create(**history)


def _upgrade_revs(script, destination, current_rev):
    with script._catch_revision_errors(
        ancestor="Destination %(end)s is not a valid upgrade "
        "target from current head(s)",
        end=destination,
    ):
        # executa os upgrades
        revs = script.revision_map.iterate_revisions(
            destination, current_rev, implicit_base=True
        )
        revs = list(revs)
        resposta = []
        histories = []

        for script_aux in reversed(list(revs)):
            resposta.append(migration.MigrationStep.upgrade_from_script(
                script.revision_map, script_aux
            ))
            histories.append({
                'titulo': script_aux.doc,
                'anterior_id': script_aux.down_revision,
                'atual_id': script_aux.revision,
                'data_criacao': datetime.strptime(
                    script_aux.longdoc.split('Date: ')[-1],
                    '%Y-%m-%d %H:%M:%S.%f'),
                'data_execucao': datetime.now(),
                'tipo': 'UPGRADE'
            })

        return (resposta, histories)



STYLE = """
<style type="text/css">
    .div-table {
        display: flex;
        flex-direction: column;
    }
    .separated-column {
        display: flex;
        flex-direction: column;
    }
    .table {
        width: 100%;
        height: auto;
        margin-top: 1px;
        border-collapse: collapse;
        text-align: center;
    }
    .tr-table-top {
        border-bottom: 0.5px solid gray;
        border-top: 0.5px solid gray;
    }
    .tr-table-center {
        border-top: 0px solid gray;
        border-bottom: 0px solid gray;
    }
    .tr-table-bottom {
        border-bottom: 0.5px solid gray;
    }
    .tr-table-right {
        border-right: 0.5px solid gray;
    }
    .tr-table-left {
        border-left: 0.5px solid gray;
    }
    .tr-table-color{
        background-color: #deebf7;
    }
    .td-table {
        text-align: center;
        padding-top: 1px;
        padding-bottom: 1px;
        border: 0.5px solid gray;
    }
    .text-primary {
        color: #324581;
        font-weight: bold;
    }
    .text-green {
        color: green;
        font-weight: bold;
    }
    .text-red {
        color: red;
        font-weight: bold;
    }
    .div-header{
        display: flex; 
        flex-direction: row; 
        justify-content: space-between; 
        align-items: center;
    }
    .sub-header-info{
        display: flex; 
        flex-direction: row; 
        justify-content: space-between;
        width: 100%;
    }
    .sub-header-text{
        display: flex;
        flex-direction: row;
        align-items: center;
        justify-content: center;
    }
    .sub-header-info-label{
        width: 10%; 
        text-align: center;
    }
</style>
"""  # noqa


from viggocore.common.subsystem import router


class Router(router.Router):

    def __init__(self, collection, routes=[]):
        super().__init__(collection, routes)

    @property
    def routes(self):
        return []



import flask
from viggocore.common import controller, utils, exception


class Controller(controller.CommonController):

    def __init__(self, manager, resource_wrap, collection_wrap):
        super(Controller, self).__init__(
            manager, resource_wrap, collection_wrap)

    # return start and end
    def _get_page_and_page_size(self, page, page_size):
        if page < 0:
            raise exception.BadRequest(
                'the page parameter must be greater than -1')
        if page_size < 1:
            raise exception.BadRequest(
                'the page_size parameter must be greater than 0')

        if page == 0:
            return (page, page_size)
        else:
            start = page * page_size
            end = start + page_size
            return (start, end)

    def get_table_names(self):
        try:
            msg = (
                'É necessário sobrescrever a variável lista_table_names' +
                'no projeto específico para garantir que as ' +
                'tabelas listadas são apenas as importantes.')
            # se não preencher a lista no projeto específico lançara a exceção
            if len(self.lista_table_names) == 0:
                return flask.Response(response=msg, status=400)
        except Exception:
            return flask.Response(response=msg,
                                  status=400)

        filters = self._filters_parse()
        filters = self._filters_cleanup(filters)
        filters = self._parse_list_options(filters)

        lista_response = self.lista_table_names
        total = len(self.lista_table_names)

        name = filters.pop('name', None)
        if name is not None:
            if '%' in name:
                name = name.replace('%', '')
                lista_response = list(
                    filter(lambda x: name in x,
                           self.lista_table_names))
            else:
                lista_response = list(
                    filter(lambda x: name == x,
                           self.lista_table_names))

        require_pagination = filters.pop('require_pagination', False)
        page = filters.pop('page', None)
        page_size = filters.pop('page_size', None)
        total = len(lista_response)

        if require_pagination and None not in [page, page_size]:
            page = int(page)
            page_size = int(page_size)

            start, end = self._get_page_and_page_size(page, page_size)

            response = {'table_names': lista_response[start:end]}

            response.update({'pagination': {'page': page,
                                            'page_size': page_size,
                                            'total': total}})
        else:
            response = {'table_names': lista_response[start:end]}
        return flask.Response(response=utils.to_json(response),
                              status=200,
                              mimetype="application/json")



TD = """
<td style="width:{width};" class="td2-table tr-table-border">
    {value}
</td>
"""


TD_BOLD = """
<td style="width:{width};" class="td2-table tr-table-border">
    <b>{value}</b>
</td>
"""


TR = """
<tr class="tr-table-center tr-table-border">
    {tds}
</tr>
"""

TR_FOOTER = """
<tr class="tr-table-center tr-table-border">
    <td style="width:0%;" class="td1-table tr-table-border">
        </br>
        <b>{count}</b>
    </td>
</tr>
"""

BODY = """
<div class="div-table">
    <table class="table">
        {tr_title}
        {trs}
    </table>
</div>
"""



# imports
import os
import requests
from datetime import datetime

from viggocore.common.subsystem.gerar_pdf.relatorio_html import (
    FILTRO_ROW_TEMPLATE, HEADER_DEFAULT, TEMPLATE_DEFAULT)
from viggocore.common import exception
from viggocore.common.subsystem.entity import DATE_FMT, DATETIME_FMT

from viggocore.common.subsystem.gerar_pdf.relatorio_styles import (
    style_1, style_2, style_3, style_4, style_5, style_6, style_7,
    style_8, style_default)
from viggocore.common.subsystem.gerar_pdf.relatorio_styles.style_paginacao \
    import STYLE_PAGINATION_TYPE
from typing import List, Dict


# constantes
HTML2PDF_URL = os.getenv('HTML2PDF_URL', 'http://172.17.0.1:1234/html2pdf')
DOMAIN_NAME_STYLE = (
    'font-weight: lighter; margin-top: 0px; margin-bottom: 0px; ' +
    'padding-top: 0px; padding-bottom: 0px;')
DOMAIN_INFO_STYLE = (
    'font-weight: lighter; margin-top: 0px; margin-bottom: 0px; ' +
    'padding-top: 0px; padding-bottom: 0px;')
TITLE_STYLE = 'font-weight: bold; font-size: 1.058rem;'


# funções
def gerar_pdf(html, parametros=None, url=HTML2PDF_URL):
    data = {
        'html': html,
        'parametros': (
            parametros if parametros is not None else get_parametros())
    }
    r = requests.post(url, json=data)
    if r.status_code != 200:
        raise exception.ViggoCoreException(r.text)
    else:
        return r.content


def format_qtd(qtd, precisao=2):
    mascara = '{:_.' + str(precisao) + 'f}'
    return mascara.format(float(qtd)).replace('_', ' ')


def format_dinheiro(valor, precisao=2, moeda=''):
    mascara = '{:_.' + str(precisao) + 'f}'
    return moeda + mascara.format(float(valor))\
        .replace('.', ',').replace('_', '.')


def get_dia_da_semana(data_hora):
    dia = data_hora.weekday()
    dia_nomes = (
        "Segunda-feira", "Terça-feira", "Quarta-feira", "Quinta-feira",
        "Sexta-feira", "Sábado", "Domingo")
    dia_nome = dia_nomes[dia]
    return dia_nome


def get_mes_por_extenso(data_hora):
    mes = data_hora.month
    mes_nomes = (
        "UNKNOWN", "Janeiro", "Fevereiro", "Março", "Abril",
        "Maio", "Junho", "Julho",
        "Agosto", "Setembro", "Outubro", "Novembro", "Dezembro")
    mes_nome = mes_nomes[mes]
    return mes_nome


def normaliza_dia_ou_mes(dia_ou_mes):
    response = str(dia_ou_mes)
    if dia_ou_mes < 10:
        response = f'0{dia_ou_mes}'
    return response


def get_data_formatada(data_hora):
    return data_hora.strftime('%d/%m/%Y')


def formatar_cep(cep):
    return f'{cep[:5]}-{cep[5:]}'


def apply_mask_cpf_cnpj(cpf_cnpj):
    cpf_cnpj_with_mask = '-'
    if cpf_cnpj is None:
        return '-'
    cpf_cnpj = cpf_cnpj.strip()
    if len(cpf_cnpj) == 11:
        cpf_cnpj_with_mask = '{}.{}.{}-{}'.\
            format(cpf_cnpj[:3], cpf_cnpj[3:6], cpf_cnpj[6:9], cpf_cnpj[9:])
    elif len(cpf_cnpj) == 14:
        cpf_cnpj_with_mask = '{}.{}.{}/{}-{}'.\
            format(cpf_cnpj[:2], cpf_cnpj[2:5], cpf_cnpj[5:8],
                   cpf_cnpj[8:12], cpf_cnpj[12:])
    return cpf_cnpj_with_mask


def get_data_solicitacao(**kwargs):
    data_solicitacao = kwargs.pop('data_solicitacao', None)
    if data_solicitacao is None:
        raise exception.BadRequest('É orbrigatório passar a data_solicitação.')
    try:
        if 'T' in data_solicitacao:
            data_solicitacao = datetime.strptime(
                data_solicitacao, DATETIME_FMT)
            data_solicitacao = data_solicitacao.strftime('%d/%m/%Y, %H:%M')
        else:
            data_solicitacao = datetime.strptime(data_solicitacao, DATE_FMT)
            data_solicitacao = data_solicitacao.strftime('%d/%m/%Y')
    except Exception:
        raise exception.BadRequest(
            'Não foi possível converter a data_solicitacao.')
    return data_solicitacao


def get_data_solicitacao_dt(**kwargs):
    data_solicitacao = kwargs.pop('data_solicitacao', None)
    if data_solicitacao is None:
        raise exception.BadRequest('É obrigatório passar a data_solicitação.')
    try:
        if 'T' in data_solicitacao:
            data_solicitacao = datetime.strptime(
                data_solicitacao, DATETIME_FMT)
        else:
            data_solicitacao = datetime.strptime(data_solicitacao, DATE_FMT)
    except Exception:
        raise exception.BadRequest(
            'Não foi possível converter a data_solicitacao.')
    return data_solicitacao


# parametros padrões para passar para o componente html2pdf
def get_parametros(margin_left=0.1, margin_right=0.1,
                   margin_top=0.1, landscape=False,
                   display_header_footer=False,
                   header_template='',
                   footer_template=''):
    '''
    link dos possíveis parametros: https://chromedevtools.github.io/devtools-protocol/tot/Page/#method-printToPDF
    '''  # noqa
    FOOTER_DEFAULT = 'o FOOTER_DEFAULT não foi implementado'
    if len(footer_template) == 0:
        footer_template = FOOTER_DEFAULT

    parametros = {
        'marginLeft': margin_left,
        'marginRight': margin_right,
        'marginTop': margin_top,
        'landscape': landscape,
        'displayHeaderFooter': display_header_footer,
        'headerTemplate': header_template,
        'footerTemplate': footer_template,
    }
    return parametros


# funcao de formatar datas que será usada nas duas próximas funções
def _format_data(data):
    data_splited = data.split('-')
    dia = data_splited[2]
    mes = data_splited[1]
    if len(dia) == 1:
        dia = '0' + dia
    if len(mes) == 1:
        mes = '0' + mes
    ano = data_splited[0]
    return f'{dia}/{mes}/{ano}'


# tratar filtros 'de' e 'ate'
def tratar_filtros_de_ate(filtros, filtros_dict, filtro_mapper_dict={}):
    has_de = 'de' in filtros.keys()
    has_ate = 'ate' in filtros.keys()
    has_attribute = 'attribute' in filtros.keys()
    if has_de and has_ate and has_attribute:
        de = filtros.get('de', '')
        ate = filtros.get('ate', '')
        attribute = filtros.get('attribute', '')

        chave = filtro_mapper_dict.get(attribute, 'NÃO MAPEADO')
        de = _format_data(de)
        ate = _format_data(ate)
        filtros_dict.append(montar_filtro_dict(
            descricao=chave,
            valor=f'entre {de} e {ate}'
        ))
    return filtros_dict


# tratar filtros de multiplas datas
def tratar_filtros_multiplas_datas(filtros, filtros_dict,
                                   filtro_mapper_dict={}):
    has_de_filter = 'de_filter' in filtros.keys()
    has_ate_filter = 'ate_filter' in filtros.keys()
    has_attributes_filter = 'attributes_filter' in filtros.keys()
    if has_de_filter and has_ate_filter and has_attributes_filter:
        de_filter = filtros.get('de_filter', '').split(',')
        ate_filter = filtros.get('ate_filter', '').split(',')
        attributes_filter = filtros.get('attributes_filter', '').split(',')
        for i in range(len(attributes_filter)):
            chave = filtro_mapper_dict.get(attributes_filter[i], 'NÃO MAPEADO')
            de = _format_data(de_filter[i])
            ate = _format_data(ate_filter[i])
            filtros_dict.append(montar_filtro_dict(
                descricao=chave,
                valor=f'entre {de} e {ate}'
            ))
    return filtros_dict


# funções usadas para preencher o html do relatório
def get_domain_name(domain):
    if domain.display_name is not None:
        return domain.display_name
    else:
        return domain.name


def get_logo_path(manager, domain_id):
    try:
        HTML2PDF_FILE_DIR = os.environ.get('HTML2PDF_FILE_DIR', '')
        IMAGEM_DEFAULT = (
            HTML2PDF_FILE_DIR + '/default/default-domain-image.png')

        # exemplo de PATH_BASE_IMAGES = '/all_files/project_name'
        PATH_BASE = os.environ.get('PATH_BASE_IMAGES', None)
        if PATH_BASE is None:
            raise exception.BadRequest(
                'É obrigatório preencher a variável PATH_BASE_IMAGES no env.')

        imagem_path = IMAGEM_DEFAULT
        domains = manager.api.domains().list(id=domain_id)

        if len(domains) > 0 and domains[0].logo_id is not None:
            path, filename = manager.api.images().get(
                id=domains[0].logo_id,
                quality='max')
            imagem_path = f'{PATH_BASE}{path}/{filename}'
        return imagem_path
    except Exception:
        return ''


def montar_filtro_dict(descricao, valor) -> Dict[str, str]:
    return {
        'descricao': str(descricao),
        'valor': str(valor)
    }


def montar_filtros(filtros: List[Dict[str, str]]) -> str:
    filtros_html = ''
    if len(filtros) > 0:
        for filtro in filtros:
            filtro_html = FILTRO_ROW_TEMPLATE.format(
                descricao=filtro.get('descricao', ''),
                valor=filtro.get('valor', '')
            )
            filtros_html += filtro_html
        filtros_html = filtros_html[:-3]
    if len(filtros_html) > 0:
        filtros_html = 'Filtros: ' + filtros_html
    return filtros_html


def montar_header(img_url, nome_empresa, titulo, data_emissao, filtros,
                  info_empresa='', domain_name_style=None,
                  domain_info_style=None, title_style=None):
    # se os styles customizados não forem passados então o backend
    # vai usar os padrões
    if domain_name_style is None:
        domain_name_style = DOMAIN_NAME_STYLE
    if domain_info_style is None:
        domain_info_style = DOMAIN_INFO_STYLE
    if title_style is None:
        title_style = TITLE_STYLE

    # info_empresa = são informações como o endereço e o doc do domain
    return HEADER_DEFAULT.format(
        img_url=img_url,
        nome_empresa=nome_empresa,
        info_empresa=info_empresa,
        titulo=titulo,
        data_emissao=data_emissao,
        filtros=filtros,
        domain_name_style=domain_name_style,
        domain_info_style=domain_info_style,
        title_style=title_style
    )


def get_style(op_style: int):
    opcoes = {
        1: style_1.STYLE,
        2: style_2.STYLE,
        3: style_3.STYLE,
        4: style_4.STYLE,
        5: style_5.STYLE,
        6: style_6.STYLE,
        7: style_7.STYLE,
        8: style_8.STYLE,
    }
    style_escolhido = opcoes.get(op_style, style_1.STYLE)
    return (style_default.STYLE + style_escolhido)


# todas as variáveis devem ser html em string
def montar_template(header: str, body: str, footer: str = '',
                    style: str = '', op_style: int = 7,
                    style_pagination: STYLE_PAGINATION_TYPE =
                    STYLE_PAGINATION_TYPE.RETRATO):
    style_final = get_style(op_style) + style
    # se quiser gerar o relatório com paginação então aplica o style da
    # paginação específico. O style definirá a direção da folha se será
    # RETRATO ou PAISAGEM
    if style_pagination is not None:
        style_final += style_pagination.value

    return TEMPLATE_DEFAULT.format(
        style=style_final,
        header=header,
        body=body,
        footer=footer
    ).replace('\n', '')



from viggocore.common import subsystem, controller
from viggocore.subsystem.project_cost \
    import resource, manager

subsystem = subsystem.Subsystem(resource=resource.ProjectCost,
                                manager=manager.Manager,
                                controller=controller.CommonController)



import os
import requests
from viggocore.common.valida_cpf_cnpj import valida_cpf_cnpj
from viggocore.common import exception
from viggocore.common.subsystem import operation


API_CPF_CONSULTA_PF = 'API_CPF_CONSULTA_PF'
API_CPF_CONSULTA_PJ = 'API_CPF_CONSULTA_PJ'
API_CPF_AUTH = 'API_CPF_CONSULTA_AUTH'


class ConsultaCpfCnpj(operation.List):
    def pre(self, **kwargs):
        # obriga a presença do cpf_cnpj
        self.cpf_cnpj = kwargs.get('cpf_cnpj', None)
        if self.cpf_cnpj is None:
            raise exception.BadRequest('ERRO! cpf_cnpj não informado')

        # remove caracteres especiais
        self.cpf_cnpj = self.cpf_cnpj \
            .replace(' ', '') \
            .replace('-', '') \
            .replace('/', '') \
            .replace('.', '')

        # valida o cpf/cnpj
        self.is_cpf_cnpj_valid = valida_cpf_cnpj(self.cpf_cnpj)
        if self.is_cpf_cnpj_valid is False:
            raise exception.BadRequest(
                'ERRO! o CPF/CNPJ informado não é válido')

        # define se o valor trata um cpf ou um cnpj
        self.is_cpf = len(self.cpf_cnpj) == 11

        return True

    def do(self, session, **kwargs):
        # busca as variáveis necessárias para a requisição
        auth = os.getenv(API_CPF_AUTH, None)
        op_api = API_CPF_CONSULTA_PF if self.is_cpf \
            else API_CPF_CONSULTA_PJ
        api = os.getenv(op_api, None)
        if not all([auth, api]):
            raise exception.BadRequest(
                ' '.join([
                    'É obrigatório preencher as variáveis de ambiente: ',
                    f'"{API_CPF_AUTH}", "{API_CPF_CONSULTA_PF}" e ',
                    f'"{API_CPF_CONSULTA_PJ}".']))

        # monta header
        headers = {
            'Authorization': f'Basic {auth}'
        }

        # monta url
        api_url = api + self.cpf_cnpj

        try:
            # tenta fazera requisição e trata o resultado
            request = requests.get(api_url, headers=headers)
            # caso o resultado seja sucesso, retorna o contaúdo da consulta
            if request.status_code == 200:
                return request.json()

        except Exception:
            # em caso de erro na requisição, dispara uma exceção com erro
            # diferente
            raise exception.BadRequest(
                'ERRO! Ocorreu uma falha na API ao consultar o CPF/CNPJ')

        # caso não seja sucesso e nem erro na requisição, dispara uma exceção
        raise exception.BadRequest(
            'ERRO! Não foi possível consultar o CPF/CNPJ')




from viggocore.common import subsystem

from viggocore.subsystem.log_system import \
    manager, resource, router


subsystem = subsystem.Subsystem(resource=resource.LogSystem,
                                manager=manager.Manager,
                                router=router.Router)



from sqlalchemy import orm, UniqueConstraint

from viggocore.database import db
from viggocore.common.subsystem import entity
from viggocore.common.utils import random_uuid
from viggocore.common import exception


class Module(entity.Entity, db.Model):

    CODE_SEQUENCE = 'module_seq'

    attributes = ['code', 'name', 'description']
    attributes += entity.Entity.attributes

    code = db.Column(db.Numeric(10), nullable=False, unique=True)
    name = db.Column(db.String(80), nullable=False, unique=True)
    description = db.Column(db.String(1000), nullable=True)

    functionalities = orm.relationship(
        'ModuleFunctionality',
        backref=orm.backref('module_functionalities'),
        cascade='delete,delete-orphan,save-update')

    def __init__(self, id, code, name, description=None,
                 active=True, created_at=None, created_by=None,
                 updated_at=None, updated_by=None, tag=None):
        super().__init__(id, active, created_at, created_by,
                         updated_at, updated_by, tag)
        self.code = code
        self.name = name
        self.description = description

    def _get_functionality(self, functionality_id):
        functionality = next((r for r in self.functionalities
                              if r.functionality_id == functionality_id), None)
        if not functionality:
            raise exception.BadRequest('ERRO! Functionality not found!')

        return functionality

    def add_functionality(self, functionality_id):
        try:
            self._get_functionality(functionality_id)
        except Exception:
            functionality = ModuleFunctionality(
                random_uuid(), self.id, functionality_id)
            self.functionalities.append(functionality)

    def rm_functionality(self, functionality_id):
        functionality = self._get_functionality(functionality_id)
        self.functionalities.remove(functionality)

    def add_functionalities(self, functionality_ids):
        for functionality_id in functionality_ids:
            self.add_functionality(functionality_id)
        return self

    def rm_functionalities(self, functionality_ids):
        for functionality_id in functionality_ids:
            self.rm_functionality(functionality_id)
        return self

    @classmethod
    def individual(cls):
        return 'module'

    @classmethod
    def collection(cls):
        return 'modules'


class ModuleFunctionality(entity.Entity, db.Model):

    attributes = ['id', 'module_id', 'functionality_id']

    module_id = db.Column(
        db.CHAR(32), db.ForeignKey('module.id'), nullable=False)
    module = orm.relationship(
        'Module',
        backref=orm.backref('module_functionality_module'),
        viewonly=True)
    functionality_id = db.Column(
        db.CHAR(32), db.ForeignKey("functionality.id"), nullable=False)
    functionality = orm.relationship(
        'Functionality',
        backref=orm.backref('module_functionality_functionality'))

    __table_args__ = (UniqueConstraint(
        'module_id', 'functionality_id',
        name=('module_functionality_module_id_functionality_id_uk')),)

    def __init__(self, id, module_id, functionality_id,
                 active=True, created_at=None, created_by=None,
                 updated_at=None, updated_by=None, tag=None):
        super().__init__(id, active, created_at, created_by,
                         updated_at, updated_by, tag)
        self.module_id = module_id
        self.functionality_id = functionality_id

    @classmethod
    def individual(cls):
        return ['module_functionality']

    @classmethod
    def collection(cls):
        return ['module_functionalities']



HEADER_DEFAULT = """
<div>
    <div style="width: 100%; text-align: center">
    <div class="div-header tr-table-bottom">
        <img src={img_url} alt="Avatar" class="avatar">
        <div>
            <h4 style="{domain_name_style}">{nome_empresa}</h4>
            <h4 style="{domain_info_style}">{info_empresa}</h4>
            <span style="{title_style}">{titulo}</span>
        </div>
        <span class="text-lighter fs-small">{data_emissao}</span>
    </div>
    <div class="margin-between-horizontal">
        {filtros}
    </div>
    </div>
</div>
"""  # noqa 501

FILTRO_ROW_TEMPLATE = "<b>{descricao}:</b> {valor} | "


TEMPLATE_DEFAULT = """
{style}
<div class="main-style">
    {header}
    {body}
    {footer}
</div>
"""  # noqa 501



import enum
import os
from sparkpost import SparkPost
from viggocore.common import exception


_HTML_EMAIL_USER_CREATED_TEMPLATE = """
    <div style="width: 100%; text-align: center">
        <h1>{app_name}</h1>
        <h2>CONFIRMAR E CRIAR SENHA</h2>
    </div>

    <p>Você acaba de ser cadastrado no portal da {app_name}.</p>
    <p><strong>Domínio:</strong> {domain_name}</p>
    <p><strong>Usuário:</strong> {username}</p>
    <p>Para ter acesso ao sistema você deve clicar no link abaixo
        para confirmar esse email e criar uma senha.</p>

    <div style="width: 100%; text-align: center">
        <a href="{reset_url}">Clique aqui para CONFIRMAR o
            email e CRIAR uma senha.</a>
    </div>
"""

_HTML_EMAIL_FORGOT_PASSWORD_TEMPLATE = """
    <div style="width: 100%; text-align: center">
        <h1>{app_name}</h1>
        <h2>Recuperar SENHA</h2>
    </div>

    # <p>Você acaba de ser cadastrado no portal da {app_name}.</p>
    # <p>Para ter acesso ao sistema você deve clicar no link abaixo
    #     para confirmar esse email e criar uma senha.</p>

    <div style="width: 100%; text-align: center">
        <a href="{reset_url}">Clique aqui para RECUPERAR a senha.</a>
    </div>
"""

_HTML_EMAIL_FORGOT_PASSWORD_ALTERNATIVE_TEMPLATE = '''
    <div style="width: 100%; text-align: center">
        <h1>{app_name}</h1>
        <h2>NOVAS CREDENCIAIS DE ACESSO</h2>
    </div>

    <p>Você solicitou a restauração de senha, então foi gerada uma nova senha aleatória no sistema e aqui está ela.</p>

    <p><strong>Senha:</strong> {password}</p>
'''  # noqa 501

_HTML_EMAIL_ACTIVATE_ACCOUNT_TEMPLATE = """
    <div style="width: 100%; text-align: center">
        <h1>{app_name}</h1>
        <h2>Ativar Conta</h2>
    </div>

    <p>Você acaba de ser cadastrado no portal da {app_name}.</p>
    <p><strong>Domínio:</strong> {domain_name}</p>
    <p><strong>Usuário:</strong> {username}</p>
    <p>Para ter acesso ao sistema você deve clicar no link abaixo
        para o cadastro.</p>

    <div style="width: 100%; text-align: center">
        <a href="{activate_url}">Clique aqui para ATIVAR a conta.</a>
    </div>
"""


class TypeEmail(enum.Enum):
    USER_CREATED = {'id': 1,
                    'template': _HTML_EMAIL_USER_CREATED_TEMPLATE,
                    'action': 'new'}
    FORGOT_PASSWORD = {'id': 2,
                       'template': _HTML_EMAIL_FORGOT_PASSWORD_TEMPLATE,
                       'action': 'recovery'}
    UPDATED_PASSWORD = {'id': 3, 'template': None}
    ACTIVATE_ACCOUNT = {'id': 4,
                        'template': _HTML_EMAIL_ACTIVATE_ACCOUNT_TEMPLATE}
    FORGOT_PASSWORLD_ALTERNATE = {
        'id': 5,
        'template': _HTML_EMAIL_FORGOT_PASSWORD_ALTERNATIVE_TEMPLATE
    }

    @staticmethod
    def value_of(value):
        return TypeEmail.__members__.get(value, None)

    @property
    def template(self):
        return self.value.get('template', None)


def get_html_reset_password(app_name, base_url, type_email,
                            token_id, domain, user):
    action = type_email.value.get('action')

    url = '{}/auth/reset/{}/{}?action={}'.format(base_url,
                                                 token_id,
                                                 domain.name,
                                                 action)

    return type_email.template.format(app_name=app_name,
                                      reset_url=url,
                                      domain_name=domain.name,
                                      username=user.name)


def get_html_activate_account(app_name, base_url, template,
                              token_id, user, domain):

    url = '{}/landing/activate/{}/{}/{}'.format(base_url,
                                                token_id,
                                                domain.id,
                                                user.id)

    return template.format(app_name=app_name, activate_url=url,
                           username=user.name, domain_name=domain.name)


def _get_variables():
    default_app_name = "VIGGOCORE"
    default_email_use_sandbox = False
    default_base_url = 'http://objetorelacional.com.br'
    default_noreply_email = 'noreply@objetorelacional.com.br'
    default_email_subject = 'VIGGOCORE - CONFIRMAR email e CRIAR senha'

    app_name = os.environ.get(
        'VIGGOCORE_APP_NAME', default_app_name)
    noreply_email = os.environ.get(
        'VIGGOCORE_NOREPLY_EMAIL', default_noreply_email)
    base_url = os.environ.get('VIGGOCORE_BASE_URL', default_base_url)
    email_subject = os.environ.get(
        'VIGGOCORE_EMAIL_SUBJECT', default_email_subject)
    email_use_sandbox = os.environ.get(
        'VIGGOCORE_EMAIL_USE_SANDBOX',
        default_email_use_sandbox) == 'True'

    return (app_name, base_url, noreply_email, email_subject,
            email_use_sandbox)


def send_email(type_email, token_id, user, domain, alternate_pass=None):
    try:
        sparkpost = SparkPost()
        (app_name, base_url, noreply_email, email_subject,
         email_use_sandbox) = _get_variables()

        if type_email in [TypeEmail.USER_CREATED, TypeEmail.FORGOT_PASSWORD]:
            html = get_html_reset_password(
                app_name, base_url, type_email, token_id, domain, user)
        elif type_email is TypeEmail.ACTIVATE_ACCOUNT:
            html = get_html_activate_account(app_name, base_url,
                                             type_email.template,
                                             token_id, user, domain)
        elif type_email is TypeEmail.UPDATED_PASSWORD:
            return None
        elif type_email is TypeEmail.FORGOT_PASSWORLD_ALTERNATE:
            html = type_email.template \
                .format(app_name=app_name, password=alternate_pass)
        else:
            raise exception.BadRequest()

        sparkpost.transmissions.send(
            use_sandbox=email_use_sandbox,
            recipients=[user.email],
            html=html,
            from_email=noreply_email,
            subject=email_subject
        )
    except Exception:
        # TODO(fdoliveira): do something here!
        pass



def valida_cpf_cnpj(cpf_cnpj):
    if len(cpf_cnpj) == 11:
        return valida_cpf(cpf_cnpj)
    elif len(cpf_cnpj) == 14:
        return valida_cnpj(cpf_cnpj)
    else:
        return False


def valida_cpf(cpf):
    cpf = cpf.replace('.', '').replace('-', '')
    digitos_verificadores = cpf[9:]
    cpf = cpf[:9]

    try:
        dig_1 = int(cpf[0]) * 1
        dig_2 = int(cpf[1]) * 2
        dig_3 = int(cpf[2]) * 3
        dig_4 = int(cpf[3]) * 4
        dig_5 = int(cpf[4]) * 5
        dig_6 = int(cpf[5]) * 6
        dig_7 = int(cpf[6]) * 7
        dig_8 = int(cpf[7]) * 8
        dig_9 = int(cpf[8]) * 9
    except IndexError:
        return False

    dig_1_ao_9_somados = (
        dig_1 + dig_2 + dig_3 + dig_4 + dig_5 + dig_6 + dig_7 + dig_8 + dig_9)

    dig_10 = dig_1_ao_9_somados % 11

    if dig_10 > 9:
        dig_10 = 0

    cpf += str(dig_10)

    dig_1 = int(cpf[0]) * 0
    dig_2 = int(cpf[1]) * 1
    dig_3 = int(cpf[2]) * 2
    dig_4 = int(cpf[3]) * 3
    dig_5 = int(cpf[4]) * 4
    dig_6 = int(cpf[5]) * 5
    dig_7 = int(cpf[6]) * 6
    dig_8 = int(cpf[7]) * 7
    dig_9 = int(cpf[8]) * 8
    dig_10 = int(cpf[9]) * 9

    dig_1_ao_10_somados = (
        dig_1 + dig_2 + dig_3 + dig_4 + dig_5 + dig_6 + dig_7 + dig_8 + dig_9 +
        dig_10)

    dig_11 = dig_1_ao_10_somados % 11

    if dig_11 > 9:
        dig_11 = 0

    cpf_validado = cpf + str(dig_11)

    return digitos_verificadores == cpf_validado[9:]


def valida_cnpj(cnpj):
    cnpj = cnpj.replace('.', '').replace('/', '').replace('-', '')
    digitos_verificadores = cnpj[12:]
    cnpj = cnpj[:12]

    try:
        dig_1 = int(cnpj[0]) * 6
        dig_2 = int(cnpj[1]) * 7
        dig_3 = int(cnpj[2]) * 8
        dig_4 = int(cnpj[3]) * 9
        dig_5 = int(cnpj[4]) * 2
        dig_6 = int(cnpj[5]) * 3
        dig_7 = int(cnpj[6]) * 4
        dig_8 = int(cnpj[7]) * 5
        dig_9 = int(cnpj[8]) * 6
        dig_10 = int(cnpj[9]) * 7
        dig_11 = int(cnpj[10]) * 8
        dig_12 = int(cnpj[11]) * 9
    except IndexError:
        return False

    dig_1_ao_12_somados = (
        dig_1 + dig_2 + dig_3 + dig_4 + dig_5 + dig_6 + dig_7 + dig_8 + dig_9 +
        dig_10 + dig_11 + dig_12)

    dig_13 = dig_1_ao_12_somados % 11

    if dig_13 > 9:
        dig_13 = 0

    cnpj += str(dig_13)

    dig_1 = int(cnpj[0]) * 5
    dig_2 = int(cnpj[1]) * 6
    dig_3 = int(cnpj[2]) * 7
    dig_4 = int(cnpj[3]) * 8
    dig_5 = int(cnpj[4]) * 9
    dig_6 = int(cnpj[5]) * 2
    dig_7 = int(cnpj[6]) * 3
    dig_8 = int(cnpj[7]) * 4
    dig_9 = int(cnpj[8]) * 5
    dig_10 = int(cnpj[9]) * 6
    dig_11 = int(cnpj[10]) * 7
    dig_12 = int(cnpj[11]) * 8
    dig_13 = int(cnpj[12]) * 9

    dig_1_ao_13_somados = (
        dig_1 + dig_2 + dig_3 + dig_4 + dig_5 + dig_6 + dig_7 + dig_8 + dig_9 +
        dig_10 + dig_11 + dig_12 + dig_13)

    dig_14 = dig_1_ao_13_somados % 11

    if dig_14 > 9:
        dig_14 = 0

    cnpj_validado = cnpj + str(dig_14)

    return digitos_verificadores == cnpj_validado[12:]



# style usado na ficha técnica do pedido
STYLE = """
<style type="text/css">
    .w-100 {
        width: 100%;
    }

    .w-50 {
        width: 50%;
    }

    .w-auto {
        width: auto;
    }

    .w-20 {
        width: 20%;
    }

    .flex {
        display: flex;
    }

    .flex-column {
        display: flex;
        flex-direction: column;
    }

    .justify-content-between {
        justify-content: space-between;
    }

    .justify-content-around {
        justify-content: space-around;
    }

    .align-items-center {
        align-items: center;
    }

    .main-style {
        font-size: 12px;
        height: 100%;
        font-family: \'Lucida Sans\', \'Lucida Sans Regular\', \'Lucida Grande\', \'Lucida Sans Unicode\', Geneva, Verdana, sans-serif;
    }

    .div-table {
        display: flex;
        flex-direction: column;
    }

    .table {
        width: 100%;
        height: auto;
        margin-top: 1px;
        border-collapse: collapse;
        text-align: center;
    }

    .tr-table-top {
        border-bottom: 0.5px solid gray;
        border-top: 0.5px solid gray;
    }

    .tr-table-center {
        border-top: 0px solid gray;
        border-bottom: 0px solid gray;
    }

    .tr-table-bottom {
        border-bottom: 0.5px solid black;
    }

    .td-table {
        text-align: center;
        font-size: 10px;
        padding-top: 1px;
        padding-bottom: 1px;
    }

    .sub-header-info-label {
        width: auto;
        text-align: start;
    }

    .margin-between-horizontal {
        margin: 1px 0px 1px 0px;
    }

    .text-underline {
        text-decoration: underline;
    }

    .text-start {
        text-align: start;
    }

    .img-container {
        text-align: center !important;
    }

    .img-card {
        width: 100%;
        height: 100%;
        max-width: 200px;
        max-height: 200px;
        object-fit: contain;
        background-color: #fff;
        border: 2px solid black;
        border-radius: 10%;
        -moz-border-radius: 10%;
        -webkit-border-radius: 10%;
        clip-path: square();
    }

    .observacao-box {
        width: auto;
        height: auto;
        border: 1.5px solid black;
        padding: 1px;
    }

    .pl-10 {
        padding-left: 5px;
    }
</style>"""  # noqa


from viggocore.common import manager, exception
from viggocore.common.subsystem import operation
from viggocore.common.subsystem.pagination import Pagination
from viggocore.subsystem.module.resource \
    import Module, ModuleFunctionality
from viggocore.subsystem.functionality.resource \
    import Functionality
from sqlalchemy import func, and_


class Create(operation.Create):

    def _get_domain_default(self):
        domains = self.manager.api.domains().list(name='default')
        if len(domains) > 0:
            return domains[0]
        else:
            raise exception.NotFound('Default domain not found.')

    def _validar_name(self, name):
        if self.manager.verify_if_exists(name=name):
            raise exception.BadRequest(
                'There is already a module with this name.')

    def pre(self, session, **kwargs):
        name = kwargs.get('name', None)
        self._validar_name(name)
        domain = self._get_domain_default()
        domain_id = domain.id
        kwargs['code'] = self.manager.api.domain_sequences().\
            get_nextval(id=domain_id, name=Module.CODE_SEQUENCE)
        return super().pre(session, **kwargs)


class AddFunctionalities(operation.Update):

    def do(self, session, **kwargs):
        functionalities = kwargs.pop("functionalities", [])
        functionality_ids = [
            r.get('functionality_id', None) for r in functionalities]
        entity = self.entity.add_functionalities(functionality_ids)
        super().do(session=session)
        return entity


class RmFunctionalities(operation.Update):

    def do(self, session, **kwargs):
        functionalities = kwargs.pop("functionalities", [])
        functionality_ids = [
            r.get('functionality_id', None) for r in functionalities]
        entity = self.entity.rm_functionalities(functionality_ids)
        super().do(session=session)
        return entity


class GetAvailableFunctionalitys(operation.List):

    def do(self, session, **kwargs):
        # remove from kwargs to not be passed in the apply filters function
        id = kwargs.pop("id", None)
        query = session.query(Functionality). \
            join(ModuleFunctionality,
                 and_(ModuleFunctionality.functionality_id == Functionality.id,
                      ModuleFunctionality.module_id == id),
                 isouter=True) \
            .filter(ModuleFunctionality.module_id.is_(None))  # noqa
        query = self.manager.apply_filters(query, Functionality, **kwargs)
        query = query.distinct()

        dict_compare = {}
        query = self.manager.apply_filters_includes(
            query, dict_compare, **kwargs)
        query = query.distinct()

        total_rows = None
        if self.manager.with_pagination(**kwargs):
            total_rows = query.count()

        pagination = Pagination.get_pagination(Functionality, **kwargs)
        if pagination.order_by is not None:
            pagination.adjust_order_by(Functionality)
        query = self.driver.apply_pagination(query, pagination)
        result = query.all()

        return (result, total_rows)


class GetSelectedFunctionalitys(operation.Get):

    def do(self, session, **kwargs):
        # remove from kwargs to not be passed in the apply filters function
        id = kwargs.pop("id", None)
        query = session.query(Functionality). \
            join(ModuleFunctionality,
                 ModuleFunctionality.functionality_id == Functionality.id). \
            join(Module,
                 ModuleFunctionality.module_id == Module.id)\
            .filter(Module.id == id)
        query = self.manager.apply_filters(query, Functionality, **kwargs)
        query = query.distinct()

        dict_compare = {}
        query = self.manager.apply_filters_includes(
            query, dict_compare, **kwargs)
        query = query.distinct()

        total_rows = None
        if self.manager.with_pagination(**kwargs):
            total_rows = query.count()

        pagination = Pagination.get_pagination(Functionality, **kwargs)
        if pagination.order_by is not None:
            pagination.adjust_order_by(Functionality)
        query = self.driver.apply_pagination(query, pagination)
        result = query.all()

        return (result, total_rows)


class VerifyIfExists(operation.List):

    def do(self, session, **kwargs):
        normalize = func.viggocore_normalize
        name = kwargs.pop('name', None)
        if name is None:
            raise exception.BadRequest('Name is required.')
        query = session.query(Module). \
            filter(normalize(getattr(Module, 'name'))
                   .ilike(normalize(name))).\
            distinct()
        result = query.all()

        if len(result) > 0:
            return True
        else:
            return False


class Manager(manager.CommonManager):

    def __init__(self, driver):
        super(Manager, self).__init__(driver)
        self.create = Create(self)
        self.add_functionalities = AddFunctionalities(self)
        self.rm_functionalities = RmFunctionalities(self)
        self.get_available_functionalities = GetAvailableFunctionalitys(self)
        self.get_selected_functionalities = GetSelectedFunctionalitys(self)
        self.verify_if_exists = VerifyIfExists(self)



from viggocore.common import subsystem
from viggocore.subsystem.module \
    import manager, resource, controller, router

subsystem = subsystem.Subsystem(resource=resource.Module,
                                controller=controller.Controller,
                                manager=manager.Manager,
                                router=router.Router)



import flask

from viggocore.common import exception
from viggocore.common.exception import BadRequest
from viggocore.subsystem.file import controller
from viggocore.subsystem.image.resource import QualityImage


class Controller(controller.Controller):

    @classmethod
    def get_bad_request(cls, cause=None):
        bad_request = BadRequest()
        bad_request.message = cause if not None else bad_request.message

        return flask.Response(response=bad_request.message,
                              status=bad_request.status)

    def get(self, id, **kwargs):
        try:
            quality = flask.request.args.get('quality', None)
            kwargs['quality'] = \
                QualityImage[quality] if quality else QualityImage.min
            folder, filename = self.manager.get(id=id, **kwargs)
            return flask.send_from_directory(folder, filename)
        except KeyError:
            return self.get_bad_request('Unknown Quality')
        except exception.ViggoCoreException as exc:
            return flask.Response(response=exc.message,
                                  status=exc.status)



import os
from viggocore.common import exception
from datetime import datetime as datetime1


def get_size(domain_id, **kwargs):  # noqa
    size = 0
    de = kwargs.get('de', None)
    ate = kwargs.get('ate', None)

    try:
        if de is not None:
            de = de.replace(tzinfo=None)
    except Exception:
        raise exception.BadRequest('Não foi possível remover o tzinfo do "de"')

    try:
        if ate is not None:
            ate = ate.replace(tzinfo=None)
    except Exception:
        raise exception.BadRequest('Não foi possível remover o tzinfo do "ate"')

    folderpath = os.environ.get('VIGGOCORE_FILE_DIR_SIZE', '')
    if len(folderpath) == 0:
        raise exception.BadRequest('VIGGOCORE_FILE_DIR_SIZE not found')

    if de is None and ate is None:
        for ele in os.scandir(folderpath):
            if '.' not in ele.name:
                for ele2 in os.scandir(f'{folderpath}/{ele.name}'):
                    if '.' not in ele2.name:
                        for ele3 in os.scandir(f'{folderpath}/{ele.name}/{ele2.name}'):  # noqa
                            if ele3.name == domain_id:
                                for ele4 in os.scandir(f'{folderpath}/{ele.name}/{ele2.name}/{ele3.name}'):  # noqa
                                    for ele5 in os.scandir(f'{folderpath}/{ele.name}/{ele2.name}/{ele3.name}/{ele4.name}'):  # noqa
                                        size += os.path.getsize(ele5)
    else:
        for ele in os.scandir(folderpath):
            if '.' not in ele.name:
                for ele2 in os.scandir(f'{folderpath}/{ele.name}'):
                    if '.' not in ele2.name:
                        for ele3 in os.scandir(f'{folderpath}/{ele.name}/{ele2.name}'):  # noqa
                            if ele3.name == domain_id:
                                for ele4 in os.scandir(f'{folderpath}/{ele.name}/{ele2.name}/{ele3.name}'):  # noqa
                                    for ele5 in os.scandir(f'{folderpath}/{ele.name}/{ele2.name}/{ele3.name}/{ele4.name}'):  # noqa
                                        ti_c = os.path.getctime(ele5)
                                        created_at = datetime1.fromtimestamp(ti_c)  # noqa
                                        validacao = ((de is None or created_at > de) and  # noqa
                                           (ate is None or created_at < ate))  # noqa
                                        print(validacao)
                                        if validacao:
                                            size += os.path.getsize(ele5)
                                        else:
                                            print(created_at)

    response_dict = {}

    # montandos as respostas
    conversoes = ['BYTES', 'KBYTES', 'MEGAS', 'GIGAS', 'TERAS']
    for conversao in conversoes:
        response_dict.update({conversao: size})
        size /= 1024

    return response_dict
